@@ -10,7 +10,11 @@ class Query(FrontendMessage):
         self.query_string = query_string
 
     def to_bytes(self):
-        return self.message_string(pack('{0}sx'.format(len(self.query_string)), self.query_string.encode('utf-8')))
+	s = self.query_string
+	if (isinstance(s,str)):
+		s = unicode(s,'utf-8')
+        encoded = s.encode('utf-8')
+        return self.message_string(pack('{0}sx'.format(len(encoded)), encoded))
 
 
 Query._message_id('Q')
